# Approaching Terminus - descent.py
#
# Copyright (C) 2014 Guillaume Tucker <guillaume@mangoz.org>
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import random
#import splat
import splat.data
import splat.sources
import splat.gen
import splat.scales
import splat.interpol
#import tools
import heartbeat

bass_notes = [('Ab-3', 8), ('Eb-4', 5), ('G-4', 3), ('Bb-3', 3)]
melody_notes = [['Ab', 'B', 'Eb', 'F', 'Ab1'],
                ['Db', 'G', 'C']]

def gen_envelope(duration, attack=0.01):
    pts = [(0.0, -100.0), (attack, 0.0, 0.0), (duration, -100.0)]
    spline = splat.interpol.spline(pts)
    env = splat.data.Fragment(channels=1, duration=duration)
    env.offset(spline.value)
    mod = splat.data.Fragment(channels=1, duration=duration)
    splat.sources.sine(mod, 8.0, (8 / duration))
    env.offset(mod)
    return env

def filter_envelope(frag, env):
    frag.amp(env)

def gen_bass(out, envelope, period, duration, notes, key):
    gen = splat.gen.OvertonesGenerator(
        frag=out,
        filters=[(filter_envelope, (envelope,))])
    s = splat.scales.LogScale(key=key)
    bass_notes_pool = []
    for note, n in notes:
        frag = splat.data.Fragment(duration=envelope.duration)
        gen.frag = frag
        gen.run(0.0, frag.duration, s[note])
        bass_notes_pool += [frag] * n
    for i in range(0, int(duration / period)):
        out.mix(random.choice(bass_notes_pool), (i * period))

def gen_melody(out, period, duration, notes, key, level):
    gen = splat.gen.TriangleGenerator(
        frag=out, filters=[splat.filters.linear_fade])
    s = splat.scales.LogScale(key=key)
    p2 = period / 2
    m = len(notes)
    rot = 3.0
    levels = [((level + rot), (level - rot)), ((level - rot), (level + rot))]
    for i in range(int(duration / period)):
        start = (i + 0.5) * period
        end = start + p2
        for n in notes:
            gen.run(start, end, s[random.choice(n)],
                    levels=random.choice(levels))
            end += p2 / m

def main(argv):
    f_out = argv[1]
    out = splat.data.Fragment()
    p1 = splat.data.Fragment.open(heartbeat.audio_input).duration
    p2 = p1 * 5 / 8
    envelope = gen_envelope(p1)
    duration = 70.0
    key = 'Ab'

    gen_bass(out, envelope, p2, duration, bass_notes, key)
    gen_melody(out, p2, duration, melody_notes, key, -23.0)

    lvl = [(0.0, -100.0), (0.8, 0.0, 0.0),
           (out.duration - 18.0, -3.0), (out.duration, -100.0)]
    out.amp(splat.interpol.spline(lvl).value)
    out.normalize()
    out.save(f_out)

    return True

if __name__ == '__main__':
    sys.exit(0 if main(sys.argv) is True else 1)
